-- Snap amount is the amount of different angles car aims to drive,
-- (360 / vehiclesnap_amount) is the difference between each direction.
-- Car will slowly turn towards such angle.
-- Default amount is 16, recommended to try with other multiples of 4.
-- You can change snapping amount and keybinds in ingame-menus.

--local function OnOffText(value)
--  if value then return {"description.VehicleSnap_enable"}
--  else return {"description.VehicleSnap_disable"} end
--end
--
--local function PlayerToggle(event)
--  local player = game.players[event.player_index]
--  global.players[event.player_index].snap = not global.players[event.player_index].snap
--  CheckDrivingState(player)
--  -- Create floating text at car position (auto-disappears after a couple of seconds)
--  player.surface.create_entity{ name = "flying-text", position = player.position,
--    text = OnOffText(global.players[event.player_index].snap) }
--  player.set_shortcut_toggled("VehicleSnap-shortcut", global.players[event.player_index].snap)
--end
--
--script.on_event("VehicleSnap-toggle", PlayerToggle)


local ACCELERATING = defines.riding.acceleration.accelerating
local BRAKING = defines.riding.acceleration.braking
local IDLING = defines.riding.acceleration.nothing
local REVERSING = defines.riding.acceleration.reversing

local LEFT  = defines.riding.direction.left
local RIGHT = defines.riding.direction.right
local STRAIGHT = defines.riding.direction.straight

-- This is ran everytime the game is changed (adding mods upgrading etc) and installed.
local function run_install()
  global.players = global.players or {}
  for i, player in (pairs(game.players)) do
    --game.players[i].print("VehicleSnap installed") -- Debug
    global.players[i] = global.players[i] or {
      snap = true,
      player_ticks = 0,
      last_orientation = 0,
      -- driving is only true if snapping is true and player is in a valid vehicle
      driving = false,
      moves = 0,
      eff_moves = 0, -- Effective tile moves from last time period
      snap_amount = 16
    }
    --local snap = settings.get_player_settings(player)["VehicleSnap_amount"].value
    if snap ~= nil then
      global.players[i].snap_amount = 8--snap
    end
    
    CheckDrivingState(player)
    --player.set_shortcut_toggled("VehicleSnap-shortcut", global.players[i].snap)
  end
  ToggleEvents(true)
end

-- Any time a new player is created run this.
script.on_event(defines.events.on_player_created, function(event)
  global.players[event.player_index] = {
    snap = true,
    player_ticks = 0,
    last_orientation = 0,
    driving = false,
    moves = 0,
    eff_moves = 0,
  }
end)


function CheckDrivingState(player)
  local pdata = global.players[player.index]
  local driving = false
  if player.vehicle and pdata.snap then -- and player.connected
    driving = (player.vehicle.type == "car")
    pdata.moves = 0
    pdata.eff_moves = 0
  end
  pdata.driving = driving
  ToggleEvents(true)
end

script.on_event(defines.events.on_player_driving_changed_state, function(event)
  CheckDrivingState(game.players[event.player_index])
end)

script.on_event(defines.events.on_player_died, function(event)
  CheckDrivingState(game.players[event.player_index])
end)

local function onPlayerChangedPosition(event)
  local pdata = global.players[event.player_index]
  if pdata and pdata.driving then
    pdata.moves = pdata.moves + 1
    
    -- Debug player speed values
    --local player = game.players[event.player_index]
    --if player.vehicle then
    -- player.surface.create_entity{ name = "flying-text", position = player.position, text = player.vehicle.speed }
    --end
  end
end

local function onTick()
  --if true or game.tick % 60 == 0 then
    local drivers = false 
    for _, player in pairs(game.connected_players) do
      local pdata = global.players[player.index]
      if pdata.driving and pdata.snap then
        if not player.vehicle then
          if player.controller_type == defines.controllers.character then
            -- Use of Space Exploration navigation satellite changes controller to godmode,
            -- after which it returns to car.
            pdata.driving = false
          else
            drivers = true
          end
        else
          drivers = true
          
          local facing = player.vehicle.orientation
          local walking = player.walking_state
          
          --local upArrow = walking.direction == defines.direction.north
          --local downArrow = walking.direction == defines.direction.south
          --local leftArrow = walking.direction == defines.direction.west
          --local rightArrow = walking.direction == defines.direction.east
          
          local acceleration = player.riding_state.acceleration
          local direction = player.riding_state.direction
          
          local upArrow = acceleration == ACCELERATING
          local downArrow = (acceleration == BRAKING) or (acceleration == REVERSING)
          local leftArrow = direction == LEFT
          local rightArrow = direction == RIGHT
          
          local toAccelerate = upArrow or downArrow or leftArrow or rightArrow
          local toREVERSE
          
          local toFace
          if upArrow then
            toFace = 0
            if (game.tick % 10 == 0) then
              player.surface.create_entity{ name = "flying-text", position = player.position, text = "UP" }
            end 
          elseif downArrow then
            toFace = .5
            if (game.tick % 10 == 0) then
              player.surface.create_entity{ name = "flying-text", position = player.position, text = "DOWN" }
            end
          elseif leftArrow then
            toFace = 0.75
            if (game.tick % 10 == 0) then
              player.surface.create_entity{ name = "flying-text", position = player.position, text = "LEFT" }
            end
          elseif rightArrow then
            toFace = 0.25
            if (game.tick % 10 == 0) then
              player.surface.create_entity{ name = "flying-text", position = player.position, text = "RIGHT" }
            end
          end
          if toFace ~= nil then
            
            --player.surface.create_entity{ name = "flying-text", position = player.position, text = toFace }
            
            
            toFace = (((toFace - facing) + 0.5) % 1) - 0.5
            
            if toFace > 0 then
              --player.vehicle.speed = player.vehicle.prototype.
              player.riding_state = {acceleration = (toAccelerate and ACCELERATING or BRAKING), direction = RIGHT}
              player.riding_state = {acceleration = acceleration, direction = RIGHT}
            else
              player.riding_state = {acceleration = (toAccelerate and ACCELERATING or BRAKING), direction = LEFT}
              player.riding_state = {acceleration = acceleration, direction = LEFT}
            end
            
          end
          pdata.last_orientation = o
        end
      end
    end
    if not drivers then
      ToggleEvents(false)
    end
  --end
end

function ToggleEvents(enable)
  global.RegisterEvents = enable
  if enable then
    script.on_event(defines.events.on_tick, onTick)
    script.on_event(defines.events.on_player_changed_position, onPlayerChangedPosition)
  else
    script.on_event(defines.events.on_tick, nil)
    script.on_event(defines.events.on_player_changed_position, nil)
  end
end

script.on_init(run_install)
script.on_configuration_changed(run_install)

script.on_load(function()
  if global.RegisterEvents then
    ToggleEvents(true)
  end
end)
